function edp = edprofile(varargin)
% EDPROFILE Create electron density profile (EDP).
%   EDP = EDPROFILE(RAWPROFILE,NSLICE) returns EDP effective-density
%   model.
%
%   Input argument:
%       RAWPROFILE: N+1 x 5, with each column representing as follows,
%               column 1: depth (z-coordinate, unit: A) of N physical 
%                   interfaces from top to bottom (last z can be any 
%                   arbitray value, only standing for substrate); 
%               column 2: dispersion (delta) above each interface. The last
%                   row means substrate;
%               column 3: absorption (beta) above each interface. The last
%                   row means substrat;
%               column 4: roughness of each interface (unit: A). The last 
%                   row can be arbitrary); 
%               column 5: profile function of each interface with 
%                   0 = error function profile
%                   1 = hyperbolic tangent profile.
%                   Profile of the last row (substrate) can be arbitrary. 
%               Note: z increases from top to bottom. N is the total
%                  number of interfaces.
%       NSLICE: total number of slice used to discribe EDP. For example,
%           NSLICE could be 1000, if the totle sample thickness is 1000A. 
%           Each slice thickness had better be as smaller as the wavelength
%           of the probing beam, i.e 1A in case of x-ray.
%
%   Output argument:
%       EDP: NSLICE+1 x 2,
%               column 1: the total number of interfaces plus last row for 
%                   the substrate; 
%               column 2: dispersion (delta);
%               column 3: absorption (beta).
%
% Reference: Tolan, M. (1999) X-Ray Scattering from Soft-Matter Thin Films,
%   Springer, P26, Effective-density Model.
%
% Copyright 2004 Zhang Jiang
% $Revision: 1.0 $  $Date: 2004/11/27 $

% --- BOF

rawProfile = varargin{1};               % N+1 x 5
nSlice = varargin{2};                   % number of total slices (M slices, M+1 slice interfaces)

zInterface = rawProfile(1:end-1,1)';    % 1 x N, physical interface positions
delta = rawProfile(:,2)';               % 1 x N+1, delta
beta = rawProfile(:,3)';                % 1 x N+1, beta
sigma = abs(rawProfile(1:end-1,4))';         % 1 x N, roughness
profileFcn = rawProfile(1:end-1,5)';    % 1 x N, interface profile

zInterface = -zInterface;               % reverse z-axis direction (pointing up)

% --- redefine roughness, preventing dividing by zero rounghess below
sigma(find(sigma==0)) = eps;                

% --- physical number of interfaces and layers excluding infinite top and substrate
nInterface = size(zInterface,2);                % equal to N
%numLayer = nInterface-1;   % equal to N-1

% --- total thickness including roughness effect (10 times of roughness)
sigmaFactor = 10;   % roughness effect to the total thickness
thick = abs(zInterface(end) - zInterface(1));   % physical thickness
totalThick = thick + sigmaFactor*(sigma(1)+sigma(end));

% --- calculate zeta position ( depth at which the upper and lower material
% are connected continuously due to the roughness profiles of interfaces)
zeta = zeros(1,nInterface+1);       % 1 x N+1
zeta(1) = Inf;                      % top
zeta(end) = -Inf;                   % bottom
zeta(2:end-1) = ( sigma(2:nInterface).*zInterface(1:nInterface-1) + ...
                  sigma(1:nInterface-1).*zInterface(2:nInterface) ) ./ ...
                ( sigma(2:nInterface) + sigma(1:nInterface-1)  );

% --- position of all the slice interfaces 1 x M+1
zSliceInterface = linspace(...
    zInterface(1)+sigmaFactor*sigma(1),...
    zInterface(end)-sigmaFactor*sigma(end),nSlice+1);
allDelta = delta;                       % 1 x N+1 include top reference and substrate
allBeta = beta;                         % 1 x N+1 include top reference and substrate
allSigma = [eps,sigma,eps];             % 1 x N+2 inlcude pseudo roughness of infinite top reference and deep substrate
allZInterface =[Inf,zInterface,-Inf];   % 1 x N+2 include pseudo interface of substrate
W = zeros(nInterface+1,nSlice+1);       % N+1 x M+1 the fraction of a certain layer at different slice positions

profileFcn = [profileFcn(1) profileFcn profileFcn(end)];    % 1 x N+2, including pesudo interface of infinite top reference and substrate

for iMat = 1:nInterface+1
    zSliceSamllIndex = find(zSliceInterface <= zeta(iMat));
    zSliceLargeIndex = find(zSliceInterface >  zeta(iMat));
    if ~ isempty(zSliceSamllIndex)
        if profileFcn(iMat) == 0        % erf profile
            W(iMat,zSliceSamllIndex) = 0.5*(1 + erf( (zSliceInterface(zSliceSamllIndex) - allZInterface(iMat+1))   / (sqrt(2)*allSigma(iMat+1)) ));
        elseif profileFcn(iMat) == 1    % tanh profile
            W(iMat,zSliceSamllIndex) = 0.5*(1 + tanh((zSliceInterface(zSliceSamllIndex) - allZInterface(iMat+1))*pi   / (2.0*sqrt(3)*allSigma(iMat+1)) ));
        end
    end
    if ~ isempty(zSliceLargeIndex)
        if profileFcn(iMat) == 0        % erf profile
            W(iMat,zSliceLargeIndex) = 0.5*(1 - erf( (zSliceInterface(zSliceLargeIndex) - allZInterface(iMat)) / (sqrt(2)*allSigma(iMat)) ));
        elseif profileFcn(iMat) == 1    % tanh profile
            W(iMat,zSliceLargeIndex) = 0.5*(1 - tanh( (zSliceInterface(zSliceLargeIndex) - allZInterface(iMat))*pi / (2.0*sqrt(3)*allSigma(iMat)) ));
        end
    end
end

% --- calculate ajusted delta and beta for all the slices including the top
% reference and substrate by matrix manipulation.
sumWCol = sum(W,1);     
deltaAllSlice = allDelta*W./sumWCol;    % 1 x M+1 delta above each zSliceInterface
betaAllSlice = allBeta*W./sumWCol;      % 1 x M+1 beta above each zSliceInterface

% --- add top reference and substrate to zSliceInterface, delta, beta;
sliceThick = abs(zSliceInterface(2)-zSliceInterface(1));
zSliceInterface = [...
    zSliceInterface(1)+sliceThick,...
    zSliceInterface,...
    zSliceInterface(end)-sliceThick];
deltaAllSlice = [...
    delta(1),...
    deltaAllSlice,...
    delta(end)];
betaAllSlice = [...
    beta(1),...
    betaAllSlice,...
    beta(end)];

% --- return EDP M+1 x 3 with 1st col slice interfaces, 2nd and 3rd column
% delta and beta above each interface
edp = [-zSliceInterface',deltaAllSlice',betaAllSlice'];  

% --- EOF