function varargout = efield(varargin)
% EFIELD Spatial distribution of x-ray excited complex electric field.
%
%   EF = EFIELD(ALPHA,PROFILE,WAVELENGTH) calculates spatial distribution 
%       of x-ray excited electric field (normalized to the incident beam)
%       in films by Parratt's recursion.
%
%   [EF,ER,ET,KZ] = EFIELD(ALPHA,PROFILE,WAVELENGTH) also returns R and T, 
%       which are the electric field amplitudes of reflected and transmitted
%       waves within  the film.
%
%   EF = EFIELD(ALPHA,PROFILE,WAVELENGTH,PLOTFLAG) also plots electric
%       field intensity (EFI) distribution map.
%
%   Input Argument:
%       ALPHA       : 1 x M list. Incident/exit angle in the reference
%               material. Unit: degree.
%       PROFILE     : N+1 x 3 matrix. Film depth, dispersion (delta) and
%               absorption (beta) profile. N is the total number of
%               interfaces. 1st column: depth (Unit: A) of each interface
%               with z-axis pointing down into the film. 2nd and 3rd 
%               columns: dispersion (delta) and absorption (beta) 
%               coefficients in the slab above the correponding interface 
%               defined in the 1st column. NOTE: PROFILE(1,2:3) are delta 
%               and beta of the reference material above interface 
%               PROFILE(1,1); PROFILE(N+1,2:3) are used as delta and beta 
%               of the substrate; PROFILE(N+1,1) can be any arbitrary 
%               number and is not used in the calculation.
%       WAVELENGTH  : Wavelength of x-ray in the vacuum. Unit: A.
%       PLOTFLAG    : Flag to plot: 'plot'.
%
%   Output Argument:
%       EF:     N+1 x M. Complex electric field.
%       ER:     N+1 x M. Complex electric field amplitude of reflected waves.
%       ET:     N+1 x M. Complex electric field amplitude of transmitted waves.
%       KZ:     N+1 x M. Complex kz within the film.
%
%   Reference:
%   1) Tolan, M. (1999) X-Ray Scattering from Soft-Matter Thin Films, Springer
%   2) Parratt, L.G. (1954). Phys. Rev. 95, 359-369
%   3) Als-Nielsen, J. (2001) Elements of Modern X-ray Physics, Wiley John & Sons
%   4) Jiang, Z. (2007) Ph.D thesis
%
%   Copyright 2005-2006 Zhang Jiang
%   $Revision: 1.0 $  $Date: 2005/05/03$
%   $Revision: 1.1 $  $Date: 2005/06/25$ : Add options to output electric
%       field amplitudes of reflected and transmitted waves, and kz within the
%       film as a function of depth and angle.
%   $Revision: 2.0 $  $Date: 2007/09/21$ : Small deviations found in ER,
%       therefore ET and EF by Tolan's method from those obtained by
%       conventional method (parratt and Als-Nielsen). Rewrite the code
%       using formulas described in Als-Nielsen's book and Jiang's thesis.
%
% See also REFCONV, PARRATT, EDPROFILE, REFRAC

if nargin ~=3 && nargin ~=4
    error('Invalid input argument.');
    return;
end
alpha   = varargin{1};              % alpha in the first medium (M points)
profile = varargin{2};              % profile of sample; N+1 points with 
                                    % 1st the media above sample and last 
                                    % point the substrate
wavelength = varargin{3};           % wavelength of beam in vacuum
plotflag = 0;   
if nargin == 4 
    if strcmp(lower(varargin{4}),'plot')
        plotflag = 1;
    else
        error('Invalid input argumnet.');
        return;
    end
end

nLayer = size(profile,1);           % N+1 layers including 'vacuum' and substrate
z = profile(:,1);                  % slice position; the last z(N+1) dos not
                                    % count, simply stands for substrate
delta = profile(:,2);
beta = profile(:,3);
k = 2*pi/wavelength;                % wave vector in vacuum

% --- construct refractive index (N+1 x 1)
n = 1-delta+i*beta;

% --- kz in the reference material
kz = n(1)*k*sin(alpha*pi/180);                          
if size(kz,2) == 1      % transfer kz (1 x M)
    kz = kz';
end

% --- get kz in vacuum (N+1 x 1)
kzInVac = sqrt(kz.^2+k^2-k^2*n(1)^2);

% --- kz in each layer (N+1 x M)
KzLayer = sqrt( (n*ones(1,size(kzInVac,2))).^2*k^2 - k^2 + (ones(size(n,1),1)*kzInVac).^2 );

% --- Fresnel coefficients (no multiple scattering) (N x M)
r = ( KzLayer(1:end-1,:) - KzLayer(2:end,:) ) ./ ( KzLayer(1:end-1,:) + KzLayer(2:end,:) );
%t = 2*KzLayer(1:end-1,:) ./ ( KzLayer(1:end-1,:) + KzLayer(2:end,:) );
t = 1+r;
% --- calcuate R (N+1 x M) and T (N+1 x M) in each layer (with multiple scattering effect)
R = complex(zeros(nLayer,length(kz)));
T = complex(zeros(nLayer,length(kz)));
R(end,:)    = 0;                        % zero relectance in (N+1)th layer (substrate)
R(end-1,:)  = r(end,:);                 % reflectance in Nth layer 
T(1,:)      = 1;
thick = diff(z);    %  thickness of each slab (1 x N-1)
thick = thick(1:end-1);
if nLayer > 2
    for j = nLayer-2:-1:1          % N-1 reflectances needed to be calculated
        R(j,:) = (r(j,:)+R(j+1,:).*exp(i*2*KzLayer(j+1,:)*thick(j)))./...
            (1+r(j,:).*R(j+1,:).*exp(i*2*KzLayer(j+1,:)*thick(j)));
    end
    for j = 2:nLayer-1
        T(j,:) = T(j-1,:).*t(j-1,:).*exp(i*KzLayer(j,:)*thick(j-1))./...
            (1+r(j-1,:).*R(j,:).*exp(i*2*KzLayer(j,:)*thick(j-1)));
    end    
end
T(end,:) = T(end-1,:).*t(end,:);

% --- calculate complex electric field
ET = complex(zeros(nLayer,length(kz)));
ER = complex(zeros(nLayer,length(kz)));
ET(1,:) = T(1,:);
ER(1,:) = R(1,:);
if nLayer > 2
    for j=2:nLayer-1
        ET(j,:) = T(j,:).*exp(-i*KzLayer(j,:)*thick(j-1));
        ER(j,:) = T(j,:).*R(j,:).*exp(i*KzLayer(j,:)*thick(j-1));
    end
end
ET(end,:) = T(end,:);
ER(end,:) = R(end,:);
ef = (ET+ER);

if plotflag == 1
    % plot EFI map
    imagesc(alpha,profile(:,1),abs(ef).^2);
    set(gca,'YDir','normal');
    xlabel('Angle (deg.)');
    ylabel('Depth (A)');
    figure(gcf);
end
if nargout == 1
    varargout{1} = ef;
elseif nargout == 4
    varargout{1} = ef;
    varargout{2} = ER;
    varargout{3} = ET;
    varargout{4} = KzLayer;
else
    error('Invalid output argument');
end
return;