function convpardata = refconv(varargin)
% REFCONV Convolutes unsmeared reflectivity curve with resolution function.
%   CONVPARDATA = REFCONV(PARDATA,RESDWAVE,RESCONSTANT,RESFACTOR,NCONVPOINT)
%
%   Format of input:
%       PARDATA: M x 2 with the 1st col qz, and 2nd reflectivity.
%       RESDWAVE: width of wavelength distribution, d(wavelength)/wavelength.
%       RESCONSTANT: constant resolution in unit of A^-1.
%       RESFACTOR (>=3): determines the qz range to do resolution
%               [<Q(i)>-RESFACOTR*resolution,<Q(i)>+RESFACTOR*resolution].
%       NCONVPOINT (>=10): number of points in the above range to convlute.
%
%   Format of output:
%       CONVPARDATA: M x 2, 1st column qz, 2nd the convoluted reflectivity.
%
% Referece: Jan Sko Pedersen, J. Appl. Cryst. (1994). 27, 36-49
%
% Copyright 2004 Zhang Jiang
% $Revision: 1.0 $  $Date: 2004/11/18 17:49:30 $
%
% See also PARRATT

pardata = varargin{1};
resDWave = varargin{2};         % d(wavelength)/(wavelength) (unitless)
resConstant = varargin{3};      % constant resolution in A^-1
resFactor = varargin{4};        
nConvPoint = varargin{5};           % total number of points

qz = pardata(:,1);
RR = pardata(:,2);

% --- calculate the varaince of each Guassian smearing function
resVar = sqrt((resDWave/(2*sqrt(2*log(2)))*qz).^2 + resConstant^2);

% --- convolution
% Interpolate/extrapolate data within the resolution range. 10 points in
% the interval of [<Q(i)>-3*resVar(i),<Q(i)>+3*resVar(i)]
convRR = RR;
if ~isempty(find(resVar>0))             % if resVar == 0, no resolution effect
    qzLength = length(qz);
    resVar(find(resVar==0)) = eps;      % in case of been divided by zero;
    gaussianNormalizationFactor = 1./(sqrt(2*pi)*resVar);
    gaussianExpFactor = 0.5./resVar.^2;
    for iQ = 1:qzLength
        convQz = linspace(qz(iQ)-resFactor*resVar(iQ),qz(iQ)+resFactor*resVar(iQ),nConvPoint);   
        interpRR = interp1(qz,RR,convQz,'linear','extrap');
        convRRWithGaussianFcn = interpRR*gaussianNormalizationFactor(iQ).*exp( -gaussianExpFactor(iQ)*( convQz -qz(iQ) ).^2  );
        convRR(iQ) = trapz(convQz,convRRWithGaussianFcn);
    end
end

% --- return convoluted reflectivity M x 2
convpardata = [qz convRR];